@@ -17,7 +17,13 @@ from gto.constants import (
     VERSIONS_PER_STAGE,
     VersionSort,
 )
-from gto.exceptions import GTOException, NotImplementedInGTO, WrongArgs
+from gto.exceptions import (
+    GTOException,
+    NotImplementedInGTO,
+    WrongArgs,
+    WrongConfig,
+)
+from gto.index import RepoIndexManager
 from gto.ui import (
     EMOJI_FAIL,
     EMOJI_GTO,
@@ -170,6 +176,12 @@ class GtoGroup(GtoCliMixin, TyperGroup):
                     with formatter.section(gettext(section)):
                         formatter.write_dl(sections[section])
 
+    # The typer overrides this with some trick dependent on environment,
+    # use click method for taking affect of Self.format_commands
+    def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
+        # pylint: disable=bad-super-call
+        super(TyperGroup, self).format_help(ctx, formatter)
+
     def get_command(self, ctx: Context, cmd_name: str) -> Optional[Command]:
         cmd = super().get_command(ctx, cmd_name)
         if cmd is not None:
@@ -232,13 +244,13 @@ option_all_commits = Option(
     False, "-A", "--all-commits", is_flag=True, help="Read all commits"
 )
 option_message = Option(
-    None, "--message", "-m", help="Message to annotate git tag with"
+    None, "--message", "-m", help="Message to annotate the Git tag with"
 )
 option_force = Option(
     False,
     "--force",
     is_flag=True,
-    help="Create a git tag even if it already exists and is in effect",
+    help="Create the Git tag even if it already exists and is in effect",
 )
 
 
@@ -258,7 +270,8 @@ def callback_simple(  # pylint: disable=inconsistent-return-statements
 option_simple = Option(
     "auto",
     "--simple",
-    help="[auto, true, false] Use simple notation, e.g. rf#prod instead of rf#prod-5",
+    help="Use simple notation, e.g. `rf#prod` instead of `rf#prod-5`"
+    " [supported values: auto, true, false]",
     callback=callback_simple,
 )
 
@@ -338,6 +351,9 @@ option_show_ref = Option(
 option_show_description = Option(
     False, "--description", is_flag=True, help="Show description", show_default=True
 )
+option_show_custom = Option(
+    False, "--custom", is_flag=True, help="Show custom metadata", show_default=True
+)
 option_json = Option(
     False,
     "--json",
@@ -365,20 +381,21 @@ option_push_tag = Option(
     is_flag=True,
     help="Push created tag automatically (experimental)",
 )
-
 option_commit = Option(
     False,
     "--commit",
     is_flag=True,
     help="Automatically commit changes due to this command (experimental)",
 )
-
 option_push_commit = Option(
     False,
     "--push",
     is_flag=True,
     help="Push created commit automatically (experimental) - will set commit=True",
 )
+option_branch = Option(
+    None, "-b", "--branch", help="Branch to commit to. Only for remote repos."
+)
 
 
 @app.callback("gto", invoke_without_command=True, no_args_is_help=True)
@@ -484,29 +501,42 @@ def annotate(
         is_flag=True,
         help="Verify artifact is committed to Git",
     ),
+    allow_same_path: bool = Option(
+        False,
+        "--allow-same-path",
+        is_flag=True,
+        help="Allow multiple artifacts use the same path",
+    ),
     label: List[str] = Option(None, "--label", help="Labels to add to artifact"),
     description: str = Option("", "-d", "--description", help="Artifact description"),
+    custom: str = Option(
+        None,
+        "-c",
+        "--custom",
+        help="Custom metadata to add to artifact",
+    ),
     commit: bool = option_commit,
     push: bool = option_push_commit,
+    branch: str = option_branch,
     # update: bool = Option(
     #     False, "-u", "--update", is_flag=True, help="Update artifact if it exists"
     # ),
 ):
-    """Update artifact metadata annotations
-
-    Examples:
-       $ gto annotate nn --type model --path models/neural_network.h5
-    """
+    """Update artifact metadata annotations."""
     gto.api.annotate(
         repo,
         name,
         type=type,
         path=path,
         must_exist=must_exist,
+        allow_same_path=allow_same_path,
         labels=label,
         description=description,
+        custom=custom,
         commit=commit,
         push=push,
+        branch=branch,
+        stdout=True,
         # update=update,
     )
 
@@ -517,13 +547,48 @@ def remove(
     name: str = arg_name,
     commit: bool = option_commit,
     push: bool = option_push_commit,
+    branch: str = option_branch,
 ):
-    """Remove the enrichment for given artifact
+    """Remove the enrichment for given artifact."""
+    gto.api.remove(
+        repo=repo, name=name, commit=commit, push=push, branch=branch, stdout=True
+    )
 
-    Examples:
-         $ gto remove nn
-    """
-    gto.api.remove(repo, name, commit, push)
+
+@gto_command(section=CommandGroups.enriching)
+def describe(
+    repo: str = option_repo,
+    name: str = arg_name,
+    rev: str = option_rev,
+    type: Optional[bool] = option_show_type,
+    path: Optional[bool] = option_show_path,
+    description: Optional[bool] = option_show_description,
+    custom: Optional[bool] = option_show_custom,
+):
+    """Display enrichments for an artifact."""
+    assert (
+        sum(bool(i) for i in (type, path, description)) <= 1  # , custom
+    ), "Can output one key only"
+    if type:
+        field = "type"
+    elif path:
+        field = "path"
+    elif description:
+        field = "description"
+    elif custom:
+        field = "custom"
+    else:
+        field = None
+
+    artifact = gto.api.describe(repo=repo, name=name, rev=rev)
+    if not artifact:
+        return
+    annotation = artifact.dict(exclude_defaults=True)
+    if field is None:
+        format_echo(annotation, "json")
+    elif field in annotation:
+        with cli_echo():
+            echo(annotation[field])
 
 
 @gto_command(section=CommandGroups.modifying)
@@ -548,20 +613,8 @@ def register(
     ),
     push: bool = option_push_tag,
 ):
-    """Create an artifact version to signify an important, published or released iteration
-
-    Examples:
-        Register new version at HEAD:
-        $ gto register nn
-
-        Register new version at a specific ref:
-        $ gto register nn abc1234
-
-        Assign version name explicitly:
-        $ gto register nn --version v1.0.0
-
-        Choose a part to bump version by:
-        $ gto register nn --bump-minor
+    """Create an artifact version to signify an important, published
+    or released iteration.
     """
     gto.api.register(
         repo=repo,
@@ -602,21 +655,7 @@ def assign(
         help="Don't register a version at specified commit",
     ),
 ):
-    """Assign stage to specific artifact version
-
-    Examples:
-        Assign "nn" to "prod" at specific ref:
-        $ gto assign nn abcd123 --stage prod
-
-        Assign specific version:
-        $ gto assign nn --version v1.0.0 --stage prod
-
-        Assign at specific ref and name version explicitly:
-        $ gto assign nn abcd123 --version v1.0.0 --stage prod
-
-        Assign without increment:
-        $ gto assign nn HEAD --stage prod --simple
-    """
+    """Assign stage to specific artifact version."""
     if ref is not None:
         name_version = version
         version = None
@@ -654,18 +693,7 @@ def deprecate(
     delete: bool = option_delete,
     push: bool = option_push_tag,
 ):
-    """Deprecate artifact, deregister a version, or unassign a stage
-
-    Examples:
-        Deprecate an artifact:
-        $ gto deprecate nn
-
-        Deprecate a version:
-        $ gto deprecate nn v0.0.1
-
-        Unassign a stage:
-        $ gto deprecate nn v0.0.1 prod
-    """
+    """Deprecate artifact, deregister a version, or unassign a stage."""
     if stage:
         gto.api.unassign(
             repo=repo,
@@ -709,12 +737,8 @@ def parse_tag(
     name: str = arg_name,
     key: Optional[str] = Option(None, "--key", help="Which key to return"),
 ):
-    """Given git tag name created by this tool, parse it and return it's parts
-
-    Examples:
-        $ gto parse-tag rf@v1.0.0
-        $ gto parse-tag rf#prod
-    """
+    """Given git tag name created by this tool, parse it and return it's
+    parts."""
     parsed = gto.api.parse_tag(name)
     if key:
         parsed = parsed[key]
@@ -731,13 +755,7 @@ def check_ref(
     event: bool = option_show_event,
     stage: bool = option_show_stage,
 ):
-    """Find out the artifact version registered/assigned with ref
-
-    Examples:
-        $ gto check-ref rf@v1.0.0
-        $ gto check-ref rf#prod --name
-        $ gto check-ref rf#prod --version
-    """
+    """Find out the artifact version registered/assigned with ref."""
     assert (
         sum(bool(i) for i in (json, event, name, version, stage)) <= 1
     ), "Only one output formatting flags is allowed"
@@ -780,19 +798,7 @@ def show(  # pylint: disable=too-many-locals
     versions_per_stage: int = option_versions_per_stage,
     sort: str = option_sort,
 ):
-    """Show the registry state, highest version, or what's assigned in stage
-
-    Examples:
-        Show the registry:
-        $ gto show
-
-        Show versions of specific artifact in registry:
-        $ gto show nn
-
-        Show greatest version or what's in stage:
-        $ gto show nn@greatest
-        $ gto show nn#prod
-    """
+    """Show the registry state, highest version, or what's assigned in stage."""
     show_options = [show_name, show_version, show_stage, show_ref]
     assert (
         sum(bool(i) for i in [json, plain] + show_options) <= 1
@@ -853,14 +859,7 @@ def history(
     plain: bool = option_plain,
     ascending: bool = option_ascending,
 ):
-    """Show a journal of registry operations
-
-    Examples:
-        $ gto history nn
-
-        Use --all-branches and --all-commits to read more than just HEAD:
-        $ gto history nn --all-commits
-    """
+    """Show a journal of registry operations."""
     assert sum(bool(i) for i in (json, plain)) <= 1, "Only one output format allowed"
     if json:
         format_echo(
@@ -910,12 +909,7 @@ def stages(
     ),
     json: bool = option_json,
 ):
-    """Print list of stages used in the registry
-
-    Examples:
-        $ gto stages
-        $ gto stages --allowed
-    """
+    """Print list of stages used in the registry."""
     result = gto.api.get_stages(repo, allowed=allowed, used=used)
     if json:
         format_echo(result, "json")
@@ -925,11 +919,7 @@ def stages(
 
 @gto_command(hidden=True)
 def print_state(repo: str = option_repo):
-    """Technical cmd: Print current registry state
-
-    Examples:
-        $ gto print-state
-    """
+    """Technical cmd: Print current registry state."""
     state = make_ready_to_serialize(
         gto.api._get_state(repo).dict()  # pylint: disable=protected-access
     )
@@ -938,53 +928,36 @@ def print_state(repo: str = option_repo):
 
 @gto_command(hidden=True)
 def print_index(repo: str = option_repo):
-    """Technical cmd: Print repo index
-
-    Examples:
-        $ gto print-index
-    """
-    index = gto.api._get_index(  # pylint: disable=protected-access
-        repo
-    ).artifact_centric_representation()
-    format_echo(index, "json")
+    """Technical cmd: Print repo index."""
+    with RepoIndexManager.from_repo(repo) as index:
+        format_echo(index.artifact_centric_representation(), "json")
 
 
-@gto_command(section=CommandGroups.enriching)
-def describe(
+@gto_command(section=CommandGroups.querying)
+def doctor(
     repo: str = option_repo,
-    name: str = arg_name,
-    rev: str = option_rev,
-    type: Optional[bool] = option_show_type,
-    path: Optional[bool] = option_show_path,
-    description: Optional[bool] = option_show_description,
+    all_commits: bool = option_all_commits,
 ):
-    """Display enrichments for an artifact
+    """Display GTO version and check the registry for problems."""
+    with cli_echo():
+        echo(f"{EMOJI_GTO} GTO Version: {gto.__version__}")
+        echo("---------------------------------")
+        try:
+            from gto.config import (  # pylint: disable=import-outside-toplevel
+                CONFIG,
+            )
 
-    Examples:
-        $ gto describe nn --rev HEAD
-        $ gto describe nn@v0.0.1
-    """
-    assert (
-        sum(bool(i) for i in (type, path, description)) <= 1
-    ), "Can output one key only"
-    infos = gto.api.describe(repo=repo, name=name, rev=rev)
-    if not infos:
-        return
-    d = infos[0].get_object().dict(exclude_defaults=True)
-    if type:
-        if "type" not in d:
-            raise WrongArgs("No type in enrichment")
-        echo(d["type"])
-    elif path:
-        if "path" not in d:
-            raise WrongArgs("No path in enrichment")
-        echo(d["path"])
-    elif description:
-        if "description" not in d:
-            raise WrongArgs("No description in enrichment")
-        echo(d["description"])
-    else:
-        format_echo(d, "json")
+            echo(CONFIG.__repr_str__("\n"))
+        except WrongConfig:
+            echo(f"{EMOJI_FAIL} Fail to parse config")
+        echo("---------------------------------")
+
+    gto.api._get_state(repo).dict()  # pylint: disable=protected-access
+    if all_commits:
+        with RepoIndexManager.from_repo(repo) as index:
+            index.artifact_centric_representation()
+    with cli_echo():
+        echo(f"{EMOJI_OK} No issues found")
 
 
 if __name__ == "__main__":
