@@ -1,14 +1,36 @@
 import os
+import sys
 from time import sleep
 from typing import Callable, Tuple
 
 import git
 import pytest
+from click.testing import Result
+from typer.testing import CliRunner
 
 import gto
+from gto.cli import app
 from gto.config import CONFIG_FILE_NAME
 
 
+class Runner:
+    def __init__(self):
+        self._runner = CliRunner(mix_stderr=False)
+
+    def invoke(self, *args, **kwargs) -> Result:
+        return self._runner.invoke(app, *args, **kwargs)
+
+
+@pytest.fixture
+def runner() -> Runner:
+    return Runner()
+
+
+@pytest.fixture(autouse=True)
+def set_recursion_limit() -> None:
+    sys.setrecursionlimit(2000)
+
+
 @pytest.fixture
 def empty_git_repo(tmp_path):
     repo = git.Repo.init(tmp_path)
