@@ -8,9 +8,9 @@ import typer
 from packaging import version
 from typer.main import get_command_from_info
 
-from gto.api import _get_index
 from gto.cli import app
 from gto.exceptions import GTOException
+from gto.index import RepoIndexManager
 from tests.conftest import Runner
 
 from .utils import check_obj
@@ -91,11 +91,6 @@ def test_commands_args_help(app_cli_cmd):
     assert not no_help, f"{no_help} cli command args do not have help!"
 
 
-def test_commands_examples(app_cli_cmd):
-    no_examples = [cmd.name for cmd in app_cli_cmd if cmd.examples is None]
-    assert not no_examples, f"{no_examples} cli command do not have examples!"
-
-
 def test_show(empty_git_repo: Tuple[git.Repo, Callable]):
     repo, write_file = empty_git_repo
     _check_successful_cmd(
@@ -128,7 +123,7 @@ def test_commands(showcase):
     )
     _check_successful_cmd(
         "show",
-        ["-r", path, "rf@greatest", "--ref"],
+        ["-r", path, "rf@latest", "--ref"],
         "rf@v1.2.4\n",
     )
     _check_successful_cmd(
@@ -151,10 +146,21 @@ def test_commands(showcase):
         ["-r", path, "rf#production", "--ref"],
         "rf@v1.2.3\n",
     )
+    _check_successful_cmd("describe", ["-r", path, "artifactnotexist"], "")
+    _check_successful_cmd("describe", ["-r", path, "rf#stagenotexist"], "")
     _check_successful_cmd("describe", ["-r", path, "rf"], EXPECTED_DESCRIBE_OUTPUT)
+    _check_successful_cmd(
+        "describe", ["-r", path, "rf#production"], EXPECTED_DESCRIBE_OUTPUT
+    )
+    _check_successful_cmd(
+        "describe", ["-r", path, "rf@latest"], EXPECTED_DESCRIBE_OUTPUT
+    )
     _check_successful_cmd(
         "describe", ["-r", path, "rf", "--path"], "models/random-forest.pkl\n"
     )
+    _check_successful_cmd("describe", ["-r", path, "rf", "--type"], "model\n")
+    _check_successful_cmd("describe", ["-r", path, "rf", "--description"], "")
+    _check_successful_cmd("describe", ["-r", path, "rf", "--custom"], "")
     # None because of random order - fix this
     _check_successful_cmd("stages", ["-r", path], None)
     # None because of output randomness and complexity
@@ -216,6 +222,11 @@ def test_commands(showcase):
         ["-r", path, "this-tag-does-not-exist"],
         "",
     )
+    _check_successful_cmd(
+        "doctor",
+        ["-r", path],
+        None,
+    )
 
 
 EXPECTED_DESCRIBE_OUTPUT_2 = """{
@@ -251,7 +262,8 @@ def test_annotate(empty_git_repo: Tuple[git.Repo, Callable]):
             "--description",
             "some description",
         ],
-        "",
+        "Updated `artifacts.yaml`",
+        _check_output_contains,
     )
     _check_successful_cmd(
         "annotate",
@@ -264,21 +276,19 @@ def test_annotate(empty_git_repo: Tuple[git.Repo, Callable]):
             "--description",
             "new description",
         ],
-        "",
+        "Updated `artifacts.yaml`",
+        _check_output_contains,
     )
-    artifact = (
-        _get_index(repo.working_dir, file=True).get_index().state[name]
-    )  # pylint: disable=protected-access
+    with RepoIndexManager.from_repo(repo.working_dir) as index:
+        artifact = index.get_index().state[name]  # pylint: disable=protected-access
     check_obj(
-        artifact,
+        artifact.dict(exclude_defaults=True),
         dict(
             type="new-type",
             path="new/path",
-            virtual=True,
             labels=["another-label", "new-label", "some-label"],
             description="new description",
         ),
-        [],
     )
     repo.index.add(["artifacts.yaml"])
     repo.index.commit("Add new artifact")
@@ -286,12 +296,19 @@ def test_annotate(empty_git_repo: Tuple[git.Repo, Callable]):
     _check_successful_cmd(
         "describe", ["-r", repo.working_dir, name], EXPECTED_DESCRIBE_OUTPUT_2
     )
-    _check_successful_cmd("remove", ["-r", repo.working_dir, name], "")
+    _check_successful_cmd(
+        "remove",
+        ["-r", repo.working_dir, name],
+        "Updated `artifacts.yaml`",
+        _check_output_contains,
+    )
     write_file(name, "new-artifact update")
     repo.index.add(["artifacts.yaml"])
     repo.index.commit("Remove new artifact")
 
-    _check_successful_cmd("describe", ["-r", repo.working_dir, name], "")
+    _check_successful_cmd(
+        "describe", ["-r", repo.working_dir, "this-artifact-doesnt-exist"], ""
+    )
 
 
 def test_register(repo_with_commit: Tuple[git.Repo, Callable]):
